@@ -61,10 +61,16 @@
 #include <sys/wait.h>
 #endif
 #ifdef G_OS_WIN32
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 #include <sys/types.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#elif defined(G_OS_WIN32)
+#include <io.h>
+#endif
 
 #include <gst/base/base.h>
 
